/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "platform.h"
#include "vfreebusy.h"
#include <mapiutil.h>
#include "mapiext.h"
#include "nameids.h"
#include "platform.h"
#include <iostream>
using namespace std;

VFreebusyConverter::VFreebusyConverter(LPADRBOOK lpAdrBook, timezone_map *mapTimeZones, LPSPropTagArray lpNamedProps, ECIConv *lpIConv, bool blCensor, bool bNoRecipients, IMailUser *lpMailUser)
	: VConverter(lpAdrBook, mapTimeZones, lpNamedProps, lpIConv, blCensor, bNoRecipients, lpMailUser)
{
}

VFreebusyConverter::~VFreebusyConverter()
{
}

HRESULT VFreebusyConverter::HrICal2MAPI(icalcomponent *lpEventRoot, icalcomponent *lpEvent, icalitem *lpPrevItem, icalitem **lppRet)
{
	return E_NOTIMPL;
}

HRESULT VFreebusyConverter::HrAddMessageClass(icalproperty_method icMethod, icalcomponent *lpicEvent, icalitem *lpIcalItem, bool bIsException, std::list<SPropValue> *lplstMsgProps)
{
	return E_NOTIMPL;
}

HRESULT VFreebusyConverter::HrAddTimes(icalcomponent *lpicEventRoot, icalcomponent *lpicEvent, icalitem *lpIcalItem)
{
	return E_NOTIMPL;
}


HRESULT VFreebusyConverter::HrMAPI2ICal(LPMESSAGE lpMessage, icalproperty_method *lpicMethod, icaltimezone **lppicTZinfo, std::string *lpstrTZid, icalcomponent **lppEvent)
{
	return E_NOTIMPL;
}

HRESULT VFreebusyConverter::HrSetTimeProperties(LPSPropValue lpMsgProps, ULONG ulMsgProps, icaltimezone *lpicTZinfo, std::string strTZid, icalcomponent *lpEvent)
{
	return E_NOTIMPL;
}

HRESULT VFreebusyConverter::HrGetFbInfo(icalcomponent *lpFbcomp, time_t *lptStart, time_t *lptEnd, std::string *lpstrUID, std::list<std::string> *lstUsers)
{
	HRESULT hr = hrSuccess;
	icalproperty *lpicProp = NULL;
	std::string strEmail;

	// DTSTART
	lpicProp = icalcomponent_get_first_property(lpFbcomp, ICAL_DTSTART_PROPERTY);
	if (lpicProp)
		*lptStart = icaltime_as_timet (icalproperty_get_dtstart (lpicProp));

	// DTEND
	lpicProp = icalcomponent_get_first_property(lpFbcomp, ICAL_DTEND_PROPERTY);
	if (lpicProp)
		*lptEnd = icaltime_as_timet (icalproperty_get_dtend (lpicProp));

	// UID
	lpicProp = icalcomponent_get_first_property(lpFbcomp, ICAL_UID_PROPERTY);
	if (lpicProp)
		*lpstrUID = icalproperty_get_uid(lpicProp);

	// ATTENDEE
	lpicProp = icalcomponent_get_first_property(lpFbcomp, ICAL_ATTENDEE_PROPERTY);
	while (lpicProp) {
		strEmail = icalproperty_get_attendee(lpicProp);
		if (stricmp(strEmail.substr(0,7).c_str(), "mailto:") == 0) {
			strEmail = strEmail.substr(7);
		}
		lstUsers->push_back(strEmail);
		lpicProp = icalcomponent_get_next_property(lpFbcomp, ICAL_ATTENDEE_PROPERTY);
	}

	return hr;
}

HRESULT VFreebusyConverter::HrFbBlock2ical(FBBlock_1 *lpsFbblk, LONG ulblocks, time_t tDtStart, time_t tDtEnd, std::string strOrganiser, std::string strUser, std::string strUID, icalcomponent **lpicFbComponent)
{
	HRESULT hr = hrSuccess;
	std::string strBuff;
	icalcomponent *lpFbComp = NULL;
	icaltimetype ittStamp;
	icalproperty *lpicProp = NULL;
	icalperiodtype icalPeriod;
	icalparameter *icalParam = NULL;
	time_t tStart = 0;
	time_t tEnd = 0;


	lpFbComp = icalcomponent_new(ICAL_VFREEBUSY_COMPONENT);
	if (!lpFbComp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}
	
	//DTSTART
	ittStamp = icaltime_from_timet_with_zone(tDtStart, false, icaltimezone_get_utc_timezone());	
	lpicProp = icalproperty_new(ICAL_DTSTART_PROPERTY);
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}
	icalproperty_set_value(lpicProp, icalvalue_new_datetime(ittStamp));
	icalcomponent_add_property(lpFbComp, lpicProp);

	//DTEND
	ittStamp = icaltime_from_timet_with_zone(tDtEnd, false, icaltimezone_get_utc_timezone());	
	lpicProp = icalproperty_new(ICAL_DTEND_PROPERTY);
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}	
	icalproperty_set_value(lpicProp, icalvalue_new_datetime(ittStamp));
	icalcomponent_add_property(lpFbComp, lpicProp);

	//DTSTAMP
	ittStamp = icaltime_from_timet_with_zone(time(NULL), false, icaltimezone_get_utc_timezone());	
	lpicProp = icalproperty_new(ICAL_DTSTAMP_PROPERTY);
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}	
	icalproperty_set_value(lpicProp, icalvalue_new_datetime(ittStamp));
	icalcomponent_add_property(lpFbComp, lpicProp);
	
	//UID
	lpicProp = icalproperty_new(ICAL_UID_PROPERTY);
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}
	icalproperty_set_uid(lpicProp, strUID.c_str());
	icalcomponent_add_property(lpFbComp, lpicProp);

	//ORGANIZER
	lpicProp = icalproperty_new(ICAL_ORGANIZER_PROPERTY);
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}	
	icalproperty_set_organizer(lpicProp, strOrganiser.c_str());
	icalcomponent_add_property(lpFbComp, lpicProp);
	
	//ATTENDEE
	strBuff = "mailto:" + strUser;
	lpicProp = icalproperty_new_attendee(strBuff.c_str());
	if (!lpicProp) {
		hr = MAPI_E_INVALID_PARAMETER;
		goto exit;
	}
	
	// param PARTSTAT
	icalParam = icalparameter_new_partstat(ICAL_PARTSTAT_ACCEPTED);
	icalproperty_add_parameter(lpicProp, icalParam);
	
	// param CUTYPE
	icalParam = icalparameter_new_cutype(ICAL_CUTYPE_INDIVIDUAL);
	icalproperty_add_parameter(lpicProp, icalParam);

	icalcomponent_add_property(lpFbComp, lpicProp);

	// add all freebusy blocks
	for(int i = 0; i < ulblocks ; i++) {
		
		// FREEBUSY
		lpicProp = icalproperty_new(ICAL_FREEBUSY_PROPERTY);
		if (!lpicProp) {
			hr = MAPI_E_INVALID_PARAMETER;
			goto exit;
		}

		RTimeToUnixTime(lpsFbblk[i].m_tmStart, &tStart);
		RTimeToUnixTime(lpsFbblk[i].m_tmEnd, &tEnd);
		icalPeriod.start = icaltime_from_timet_with_zone(tStart, false, icaltimezone_get_utc_timezone());
		icalPeriod.end = icaltime_from_timet_with_zone(tEnd, false, icaltimezone_get_utc_timezone());
	
		icalproperty_set_freebusy(lpicProp, icalPeriod);

		switch (lpsFbblk[i].m_fbstatus)
		{	
		case fbBusy:
			icalParam = icalparameter_new_fbtype (ICAL_FBTYPE_BUSY);
			break;
		case fbTentative:
			icalParam = icalparameter_new_fbtype (ICAL_FBTYPE_BUSYTENTATIVE);
			break;
		case fbOutOfOffice:
			icalParam = icalparameter_new_fbtype (ICAL_FBTYPE_BUSYUNAVAILABLE);
			break;
		default:
			icalParam = icalparameter_new_fbtype (ICAL_FBTYPE_FREE);
			break;
		}
		
		icalproperty_add_parameter (lpicProp, icalParam);
		icalcomponent_add_property(lpFbComp,lpicProp);
	}
	
	icalcomponent_end_component(lpFbComp, ICAL_VFREEBUSY_COMPONENT);
	*lpicFbComponent = lpFbComp;

exit:

	return hr;
}
